package com.example.e13roomshoppinglist

import android.app.Activity
import android.content.Context
import android.content.ContextWrapper
import com.google.android.gms.ads.AdError
import com.google.android.gms.ads.AdRequest
import com.google.android.gms.ads.FullScreenContentCallback
import com.google.android.gms.ads.LoadAdError
import com.google.android.gms.ads.interstitial.InterstitialAd
import com.google.android.gms.ads.interstitial.InterstitialAdLoadCallback
import com.google.android.gms.ads.rewarded.RewardItem
import com.google.android.gms.ads.rewarded.RewardedAd
import com.google.android.gms.ads.rewarded.RewardedAdLoadCallback


const val interstitialUnitId = "ca-app-pub-3940256099942544/1033173712"
const val rewardedUnitId = "ca-app-pub-3940256099942544/5224354917"
var mInterstitialAd: InterstitialAd? = null
var mRewardedAd: RewardedAd? = null


fun loadInterstitial(
    context: Context,
    onLoadedAd: () -> Unit = {},
    onFailedToLoadAd: () -> Unit = {}
) {
    InterstitialAd.load(
        context,
        interstitialUnitId, //Change this with your own AdUnitID!
        AdRequest.Builder().build(),
        object : InterstitialAdLoadCallback() {
            override fun onAdFailedToLoad(adError: LoadAdError) {
                mInterstitialAd = null
                onFailedToLoadAd()
            }

            override fun onAdLoaded(interstitialAd: InterstitialAd) {
                mInterstitialAd = interstitialAd
                onLoadedAd()
            }
        }
    )
}

fun loadRewarded(
    context: Context,
    onLoadedAd: () -> Unit = {},
    onFailedToLoadAd: () -> Unit = {}
) {
    RewardedAd.load(
        context,
        rewardedUnitId, //Change this with your own AdUnitID!
        AdRequest.Builder().build(),
        object : RewardedAdLoadCallback() {
            override fun onAdFailedToLoad(adError: LoadAdError) {
                mRewardedAd = null
                onFailedToLoadAd()
            }

            override fun onAdLoaded(rewardedAd: RewardedAd) {
                mRewardedAd = rewardedAd
                onLoadedAd()
            }
        }
    )
}


fun showInterstitial(
    context: Context,
    onLoadedAd: () -> Unit = {},
    onShowingAd: () -> Unit = {},
    onAdDismissed: () -> Unit = {},
    onFailedToLoadAd: () -> Unit = {},
    onFailedToShowFullScreenAd: () -> Unit = {},
) {
    val activity = context.findActivity()

    if (mInterstitialAd != null && activity != null) {
        mInterstitialAd?.fullScreenContentCallback = object : FullScreenContentCallback() {
            override fun onAdFailedToShowFullScreenContent(e: AdError) {
                mInterstitialAd = null
                onFailedToShowFullScreenAd()
            }

            override fun onAdDismissedFullScreenContent() {
                mInterstitialAd = null

                loadInterstitial(context, onFailedToLoadAd, onLoadedAd)
                onAdDismissed()
            }
        }
        mInterstitialAd?.show(activity).also {
            onShowingAd()
        }
    }
}

fun showRewarded(
    context: Context,
    onLoadedAd: () -> Unit = {},
    onShowingAd: () -> Unit = {},
    onAdDismissed: () -> Unit = {},
    onUserEarnedRewardListener: (item: RewardItem) -> Unit = {},
    onFailedToLoadAd: () -> Unit = {},
    onFailedToShowFullScreenAd: () -> Unit = {},
) {
    val activity = context.findActivity()
    if (mRewardedAd != null && activity != null) {
        mRewardedAd?.fullScreenContentCallback = object : FullScreenContentCallback() {
            override fun onAdFailedToShowFullScreenContent(e: AdError) {
                mRewardedAd = null
                onFailedToShowFullScreenAd()
            }

            override fun onAdDismissedFullScreenContent() {
                mRewardedAd = null

                loadRewarded(context, onFailedToLoadAd, onLoadedAd)
                onAdDismissed()
            }
        }
        mRewardedAd?.show(activity, onUserEarnedRewardListener).also {
            onShowingAd()
        }
    }
}


fun removeInterstitial() {
    mInterstitialAd?.fullScreenContentCallback = null
    mInterstitialAd = null
}

fun removeRewarded() {
    mRewardedAd?.fullScreenContentCallback = null
    mRewardedAd = null
}


fun Context.findActivity(): Activity? = when (this) {
    is Activity -> this
    is ContextWrapper -> baseContext.findActivity()
    else -> null
}
